#ifndef SCENETREE_H
#define SCENETREE_H

#include <vector>
#include <string>
#include "scene.h"
#include "light.h"
#include "resourcemanager.h"

class SimplePipeline;

/**
 * @brief The SceneNode class represents a class of the game
 * that will be updated or used for rendering every frame
 */
class SceneNode
{
public:
    SceneNode* m_parent;
    virtual void update() = 0;
};

/**
 * @brief The ContainerNode class represents a node which main use is to contain a collection of nodes
 */
class ContainerNode : public SceneNode
{
public:
    virtual void update();

    /**
     * @brief addChild adds node in the ContainerNode's children list
     */
    void addChild(SceneNode* node);

    /**
     * @brief removeChild removes the first iteration of node of the children list
     */
    void removeChild(SceneNode* node);
protected:
    std::vector<SceneNode*> m_children;
};

/**
 * @brief The CameraNode class is a scene node that can be used by the renderer
 */
class CameraNode : public Camera, SceneNode
{

};

class SceneTree : public Scene
{
public:
    SceneTree();
    ~SceneTree();

    virtual SceneIterator<Light*>* getLights();
    virtual SceneIterator<GeometryNode*>* getGeometry();

    void update();

    Texture* getSkybox() {return m_skybox;}
    void setSkybox(Texture* skybox) {m_skybox = skybox;}

    void setMainCamera(CameraNode *camera);
    void addRootObject(SceneNode *node) {m_root.addChild(node);}

private:
    ContainerNode m_root;
    Texture* m_skybox;
    SimplePipeline* m_simplepipeline;
};

// Additionnal Nodes :



#endif // SCENETREE_H
